@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
+import cgi
 import os.path
 import struct
 import sys
@@ -320,6 +321,8 @@ def select_app(environ, start_response):
     headers = [
         ('Content-Type', 'text/html; charset=utf-8'),
         ('Content-Length', str(len(body)))]
+    # PEP 3333 requires native strings:
+    headers = [(str(k), str(v)) for k, v in headers]
     start_response(status, headers)
     return [body]
 
@@ -372,6 +375,8 @@ def select_app_without_values(environ, start_response):
     headers = [
         ('Content-Type', 'text/html; charset=utf-8'),
         ('Content-Length', str(len(body)))]
+    # PEP 3333 requires native strings:
+    headers = [(str(k), str(v)) for k, v in headers]
     start_response(status, headers)
     return [body]
 
@@ -424,6 +429,8 @@ def select_app_without_default(environ, start_response):
     headers = [
         ('Content-Type', 'text/html; charset=utf-8'),
         ('Content-Length', str(len(body)))]
+    # PEP 3333 requires native strings:
+    headers = [(str(k), str(v)) for k, v in headers]
     start_response(status, headers)
     return [body]
 
@@ -475,6 +482,8 @@ def select_app_unicode(environ, start_response):
     headers = [
         ('Content-Type', 'text/html; charset=utf-8'),
         ('Content-Length', str(len(body)))]
+    # PEP 3333 requires native strings:
+    headers = [(str(k), str(v)) for k, v in headers]
     start_response(status, headers)
     if not isinstance(body, binary_type):
         raise AssertionError('Body is not %s' % binary_type)
@@ -735,6 +744,7 @@ class TestSelect(unittest.TestCase):
         display = multiple_form.submit("button")
         self.assertIn("<p>You selected </p>", display, display)
 
+
 class SingleUploadFileApp(object):
 
     body = b"""
@@ -744,6 +754,7 @@ class SingleUploadFileApp(object):
         <form method="POST" id="file_upload_form"
               enctype="multipart/form-data">
             <input name="file-field" type="file" value="some/path/file.txt" />
+            <input name="int-field" type="text" value="" />
             <input name="button" type="submit" value="single">
         </form>
     </body>
@@ -767,6 +778,8 @@ class SingleUploadFileApp(object):
         headers = [
             ('Content-Type', 'text/html; charset=utf-8'),
             ('Content-Length', str(len(body)))]
+        # PEP 3333 requires native strings:
+        headers = [(str(k), str(v)) for k, v in headers]
         start_response(status, headers)
         assert(isinstance(body, binary_type))
         return [body]
@@ -776,9 +789,12 @@ class SingleUploadFileApp(object):
         uploaded_files = [(k, v) for k, v in req.POST.items() if 'file' in k]
         uploaded_files = sorted(uploaded_files)
         for name, uploaded_file in uploaded_files:
-            filename = to_bytes(uploaded_file.filename)
-            value = to_bytes(uploaded_file.value, 'ascii')
-            content_type = to_bytes(uploaded_file.type, 'ascii')
+            if isinstance(uploaded_file, cgi.FieldStorage):
+                filename = to_bytes(uploaded_file.filename)
+                value = to_bytes(uploaded_file.value, 'ascii')
+                content_type = to_bytes(uploaded_file.type, 'ascii')
+            else:
+                filename = value = content_type = b''
             file_parts.append(b"""
         <p>You selected '""" + filename + b"""'</p>
         <p>with contents: '""" + value + b"""'</p>
@@ -936,6 +952,25 @@ class TestFileUpload(unittest.TestCase):
         self.assertFile(uploaded_file1_name, uploaded_file1_contents, display)
         self.assertFile(uploaded_file1_name, uploaded_file1_contents, display)
 
+    def test_post_int(self):
+        binary_data = struct.pack(str('255h'), *range(0, 255))
+        app = webtest.TestApp(SingleUploadFileApp())
+        res = app.get('/')
+        single_form = res.forms["file_upload_form"]
+        single_form.set("file-field", ('my_file.dat', binary_data))
+        single_form.set("int-field", 100)
+        # just check it does not raise
+        single_form.submit("button")
+
+    def test_invalid_types(self):
+        binary_data = struct.pack(str('255h'), *range(0, 255))
+        app = webtest.TestApp(SingleUploadFileApp())
+        res = app.get('/')
+        single_form = res.forms["file_upload_form"]
+        single_form.set("file-field", ('my_file.dat', binary_data))
+        single_form.set("int-field", SingleUploadFileApp())
+        self.assertRaises(ValueError, single_form.submit, "button")
+
     def test_upload_invalid_content(self):
         app = webtest.TestApp(SingleUploadFileApp())
         res = app.get('/')
